@@ -3,6 +3,7 @@ package sqlmock
 import (
 	"database/sql"
 	"fmt"
+	"strconv"
 	"sync"
 	"testing"
 	"time"
@@ -11,7 +12,10 @@ import (
 func cancelOrder(db *sql.DB, orderID int) error {
 	tx, _ := db.Begin()
 	_, _ = tx.Query("SELECT * FROM orders {0} FOR UPDATE", orderID)
-	_ = tx.Rollback()
+	err := tx.Rollback()
+	if err != nil {
+		return err
+	}
 	return nil
 }
 
@@ -214,7 +218,7 @@ func TestTransactionExpectations(t *testing.T) {
 
 	err = tx.Commit()
 	if err != nil {
-		t.Errorf("an error '%s' was not expected when commiting a transaction", err)
+		t.Errorf("an error '%s' was not expected when committing a transaction", err)
 	}
 
 	// begin and rollback
@@ -350,6 +354,43 @@ func TestPreparedQueryExecutions(t *testing.T) {
 	}
 }
 
+func TestUnorderedPreparedQueryExecutions(t *testing.T) {
+	t.Parallel()
+	db, mock, err := New()
+	if err != nil {
+		t.Errorf("an error '%s' was not expected when opening a stub database connection", err)
+	}
+	defer db.Close()
+
+	mock.MatchExpectationsInOrder(false)
+
+	mock.ExpectPrepare("SELECT (.+) FROM articles WHERE id = ?").
+		ExpectQuery().
+		WithArgs(5).
+		WillReturnRows(NewRows([]string{"id", "title"}).FromCSVString("5,The quick brown fox"))
+	mock.ExpectPrepare("SELECT (.+) FROM authors WHERE id = ?").
+		ExpectQuery().
+		WithArgs(1).
+		WillReturnRows(NewRows([]string{"id", "title"}).FromCSVString("1,Betty B."))
+
+	var id int
+	var name string
+
+	stmt, err := db.Prepare("SELECT id, name FROM authors WHERE id = ?")
+	if err != nil {
+		t.Errorf("error '%s' was not expected while creating a prepared statement", err)
+	}
+
+	err = stmt.QueryRow(1).Scan(&id, &name)
+	if err != nil {
+		t.Errorf("error '%s' was not expected querying row from statement and scanning", err)
+	}
+
+	if name != "Betty B." {
+		t.Errorf("expected mocked name to be 'Betty B.', but got '%s' instead", name)
+	}
+}
+
 func TestUnexpectedOperations(t *testing.T) {
 	t.Parallel()
 	db, mock, err := New()
@@ -394,7 +435,7 @@ func TestWrongExpectations(t *testing.T) {
 		WithArgs(5).
 		WillReturnRows(rs1)
 
-	mock.ExpectCommit().WillReturnError(fmt.Errorf("deadlock occured"))
+	mock.ExpectCommit().WillReturnError(fmt.Errorf("deadlock occurred"))
 	mock.ExpectRollback() // won't be triggered
 
 	var id int
@@ -418,7 +459,7 @@ func TestWrongExpectations(t *testing.T) {
 
 	err = tx.Commit()
 	if err == nil {
-		t.Error("a deadlock error was expected when commiting a transaction", err)
+		t.Error("a deadlock error was expected when committing a transaction", err)
 	}
 
 	if err := mock.ExpectationsWereMet(); err == nil {
@@ -658,3 +699,367 @@ func ExampleSqlmock_goroutines() {
 	}
 	// Output:
 }
+
+// False Positive - passes despite mismatched Exec
+// see #37 issue
+func TestRunExecsWithOrderedShouldNotMeetAllExpectations(t *testing.T) {
+	db, dbmock, _ := New()
+	dbmock.ExpectExec("THE FIRST EXEC")
+	dbmock.ExpectExec("THE SECOND EXEC")
+
+	_, _ = db.Exec("THE FIRST EXEC")
+	_, _ = db.Exec("THE WRONG EXEC")
+
+	err := dbmock.ExpectationsWereMet()
+	if err == nil {
+		t.Fatal("was expecting an error, but there wasn't any")
+	}
+}
+
+// False Positive - passes despite mismatched Exec
+// see #37 issue
+func TestRunQueriesWithOrderedShouldNotMeetAllExpectations(t *testing.T) {
+	db, dbmock, _ := New()
+	dbmock.ExpectQuery("THE FIRST QUERY")
+	dbmock.ExpectQuery("THE SECOND QUERY")
+
+	_, _ = db.Query("THE FIRST QUERY")
+	_, _ = db.Query("THE WRONG QUERY")
+
+	err := dbmock.ExpectationsWereMet()
+	if err == nil {
+		t.Fatal("was expecting an error, but there wasn't any")
+	}
+}
+
+func TestRunExecsWithExpectedErrorMeetsExpectations(t *testing.T) {
+	db, dbmock, _ := New()
+	dbmock.ExpectExec("THE FIRST EXEC").WillReturnError(fmt.Errorf("big bad bug"))
+	dbmock.ExpectExec("THE SECOND EXEC").WillReturnResult(NewResult(0, 0))
+
+	_, _ = db.Exec("THE FIRST EXEC")
+	_, _ = db.Exec("THE SECOND EXEC")
+
+	err := dbmock.ExpectationsWereMet()
+	if err != nil {
+		t.Fatalf("all expectations should be met: %s", err)
+	}
+}
+
+func TestRunQueryWithExpectedErrorMeetsExpectations(t *testing.T) {
+	db, dbmock, _ := New()
+	dbmock.ExpectQuery("THE FIRST QUERY").WillReturnError(fmt.Errorf("big bad bug"))
+	dbmock.ExpectQuery("THE SECOND QUERY").WillReturnRows(NewRows([]string{"col"}).AddRow(1))
+
+	_, _ = db.Query("THE FIRST QUERY")
+	_, _ = db.Query("THE SECOND QUERY")
+
+	err := dbmock.ExpectationsWereMet()
+	if err != nil {
+		t.Fatalf("all expectations should be met: %s", err)
+	}
+}
+
+func TestEmptyRowSet(t *testing.T) {
+	t.Parallel()
+	db, mock, err := New()
+	if err != nil {
+		t.Errorf("an error '%s' was not expected when opening a stub database connection", err)
+	}
+	defer db.Close()
+
+	rs := NewRows([]string{"id", "title"})
+
+	mock.ExpectQuery("SELECT (.+) FROM articles WHERE id = ?").
+		WithArgs(5).
+		WillReturnRows(rs)
+
+	rows, err := db.Query("SELECT (.+) FROM articles WHERE id = ?", 5)
+	if err != nil {
+		t.Errorf("error '%s' was not expected while retrieving mock rows", err)
+	}
+
+	defer func() {
+		if er := rows.Close(); er != nil {
+			t.Error("Unexpected error while trying to close rows")
+		}
+	}()
+
+	if rows.Next() {
+		t.Error("expected no rows but got one")
+	}
+
+	err = mock.ExpectationsWereMet()
+	if err != nil {
+		t.Fatalf("all expectations should be met: %s", err)
+	}
+}
+
+// Based on issue #50
+func TestPrepareExpectationNotFulfilled(t *testing.T) {
+	t.Parallel()
+	db, mock, err := New()
+	if err != nil {
+		t.Errorf("an error '%s' was not expected when opening a stub database connection", err)
+	}
+	defer db.Close()
+
+	mock.ExpectPrepare("^BADSELECT$")
+
+	if _, err := db.Prepare("SELECT"); err == nil {
+		t.Fatal("prepare should not match expected query string")
+	}
+
+	if err := mock.ExpectationsWereMet(); err == nil {
+		t.Errorf("was expecting an error, since prepared statement query does not match, but there was none")
+	}
+}
+
+func TestRollbackThrow(t *testing.T) {
+	// Open new mock database
+	db, mock, err := New()
+	if err != nil {
+		fmt.Println("error creating mock database")
+		return
+	}
+	// columns to be used for result
+	columns := []string{"id", "status"}
+	// expect transaction begin
+	mock.ExpectBegin()
+	// expect query to fetch order, match it with regexp
+	mock.ExpectQuery("SELECT (.+) FROM orders (.+) FOR UPDATE").
+		WithArgs(1).
+		WillReturnRows(NewRows(columns).AddRow(1, 1))
+	// expect transaction rollback, since order status is "cancelled"
+	mock.ExpectRollback().WillReturnError(fmt.Errorf("rollback failed"))
+
+	// run the cancel order function
+	someOrderID := 1
+	// call a function which executes expected database operations
+	err = cancelOrder(db, someOrderID)
+	if err == nil {
+		t.Error("an error was expected when rolling back transaction, but got none")
+	}
+
+	// ensure all expectations have been met
+	if err = mock.ExpectationsWereMet(); err != nil {
+		t.Errorf("unmet expectation error: %s", err)
+	}
+	// Output:
+}
+
+func TestUnexpectedCommit(t *testing.T) {
+	// Open new mock database
+	db, mock, err := New()
+	if err != nil {
+		fmt.Println("error creating mock database")
+		return
+	}
+	mock.ExpectBegin()
+	tx, _ := db.Begin()
+	if err := tx.Commit(); err == nil {
+		t.Error("an error was expected when calling commit, but got none")
+	}
+}
+
+func TestUnexpectedCommitOrder(t *testing.T) {
+	// Open new mock database
+	db, mock, err := New()
+	if err != nil {
+		fmt.Println("error creating mock database")
+		return
+	}
+	mock.ExpectBegin()
+	mock.ExpectRollback().WillReturnError(fmt.Errorf("Rollback failed"))
+	tx, _ := db.Begin()
+	if err := tx.Commit(); err == nil {
+		t.Error("an error was expected when calling commit, but got none")
+	}
+}
+
+func TestExpectedCommitOrder(t *testing.T) {
+	// Open new mock database
+	db, mock, err := New()
+	if err != nil {
+		fmt.Println("error creating mock database")
+		return
+	}
+	mock.ExpectCommit().WillReturnError(fmt.Errorf("Commit failed"))
+	if _, err := db.Begin(); err == nil {
+		t.Error("an error was expected when calling begin, but got none")
+	}
+}
+
+func TestUnexpectedRollback(t *testing.T) {
+	// Open new mock database
+	db, mock, err := New()
+	if err != nil {
+		fmt.Println("error creating mock database")
+		return
+	}
+	mock.ExpectBegin()
+	tx, _ := db.Begin()
+	if err := tx.Rollback(); err == nil {
+		t.Error("an error was expected when calling rollback, but got none")
+	}
+}
+
+func TestUnexpectedRollbackOrder(t *testing.T) {
+	// Open new mock database
+	db, mock, err := New()
+	if err != nil {
+		fmt.Println("error creating mock database")
+		return
+	}
+	mock.ExpectBegin()
+
+	tx, _ := db.Begin()
+	if err := tx.Rollback(); err == nil {
+		t.Error("an error was expected when calling rollback, but got none")
+	}
+}
+
+func TestPrepareExec(t *testing.T) {
+	// Open new mock database
+	db, mock, err := New()
+	if err != nil {
+		fmt.Println("error creating mock database")
+		return
+	}
+	defer db.Close()
+	mock.ExpectBegin()
+	ep := mock.ExpectPrepare("INSERT INTO ORDERS\\(ID, STATUS\\) VALUES \\(\\?, \\?\\)")
+	for i := 0; i < 3; i++ {
+		ep.ExpectExec().WillReturnResult(NewResult(1, 1))
+	}
+	mock.ExpectCommit()
+	tx, _ := db.Begin()
+	stmt, err := tx.Prepare("INSERT INTO ORDERS(ID, STATUS) VALUES (?, ?)")
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer stmt.Close()
+	for i := 0; i < 3; i++ {
+		_, err := stmt.Exec(i, "Hello"+strconv.Itoa(i))
+		if err != nil {
+			t.Fatal(err)
+		}
+	}
+	tx.Commit()
+	if err := mock.ExpectationsWereMet(); err != nil {
+		t.Errorf("there were unfulfilled expections: %s", err)
+	}
+}
+
+func TestPrepareQuery(t *testing.T) {
+	// Open new mock database
+	db, mock, err := New()
+	if err != nil {
+		fmt.Println("error creating mock database")
+		return
+	}
+	defer db.Close()
+	mock.ExpectBegin()
+	ep := mock.ExpectPrepare("SELECT ID, STATUS FROM ORDERS WHERE ID = \\?")
+	ep.ExpectQuery().WithArgs(101).WillReturnRows(NewRows([]string{"ID", "STATUS"}).AddRow(101, "Hello"))
+	mock.ExpectCommit()
+	tx, _ := db.Begin()
+	stmt, err := tx.Prepare("SELECT ID, STATUS FROM ORDERS WHERE ID = ?")
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer stmt.Close()
+	rows, err := stmt.Query(101)
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer rows.Close()
+	for rows.Next() {
+		var (
+			id     int
+			status string
+		)
+		if rows.Scan(&id, &status); id != 101 || status != "Hello" {
+			t.Fatal("wrong query results")
+		}
+
+	}
+	tx.Commit()
+	if err := mock.ExpectationsWereMet(); err != nil {
+		t.Errorf("there were unfulfilled expections: %s", err)
+	}
+}
+
+func TestExpectedCloseError(t *testing.T) {
+	// Open new mock database
+	db, mock, err := New()
+	if err != nil {
+		fmt.Println("error creating mock database")
+		return
+	}
+	mock.ExpectClose().WillReturnError(fmt.Errorf("Close failed"))
+	if err := db.Close(); err == nil {
+		t.Error("an error was expected when calling close, but got none")
+	}
+	if err := mock.ExpectationsWereMet(); err != nil {
+		t.Errorf("there were unfulfilled expections: %s", err)
+	}
+}
+
+func TestExpectedCloseOrder(t *testing.T) {
+	// Open new mock database
+	db, mock, err := New()
+	if err != nil {
+		fmt.Println("error creating mock database")
+		return
+	}
+	defer db.Close()
+	mock.ExpectClose().WillReturnError(fmt.Errorf("Close failed"))
+	db.Begin()
+	if err := mock.ExpectationsWereMet(); err == nil {
+		t.Error("expected error on ExpectationsWereMet")
+	}
+}
+
+func TestExpectedBeginOrder(t *testing.T) {
+	// Open new mock database
+	db, mock, err := New()
+	if err != nil {
+		fmt.Println("error creating mock database")
+		return
+	}
+	mock.ExpectBegin().WillReturnError(fmt.Errorf("Begin failed"))
+	if err := db.Close(); err == nil {
+		t.Error("an error was expected when calling close, but got none")
+	}
+}
+
+func TestPreparedStatementCloseExpectation(t *testing.T) {
+	// Open new mock database
+	db, mock, err := New()
+	if err != nil {
+		fmt.Println("error creating mock database")
+		return
+	}
+	defer db.Close()
+
+	ep := mock.ExpectPrepare("INSERT INTO ORDERS").WillBeClosed()
+	ep.ExpectExec().WillReturnResult(NewResult(1, 1))
+
+	stmt, err := db.Prepare("INSERT INTO ORDERS(ID, STATUS) VALUES (?, ?)")
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	if _, err := stmt.Exec(1, "Hello"); err != nil {
+		t.Fatal(err)
+	}
+
+	if err := stmt.Close(); err != nil {
+		t.Fatal(err)
+	}
+
+	if err := mock.ExpectationsWereMet(); err != nil {
+		t.Errorf("there were unfulfilled expections: %s", err)
+	}
+}
